@@ -4,10 +4,13 @@
 #' aesthetic is most commonly used for points and text, and humans perceive
 #' the area of points (not their radius), so this provides for optimal
 #' perception. `scale_size_area` ensures that a value of 0 is mapped
-#' to a size of 0.
+#' to a size of 0. `scale_size_binned` is a binned version of `scale_size` that
+#' scales by area (but does not ensure 0 equals an area of zero). For a binned
+#' equivalent of `scale_size_area` use `scale_size_binned_area`.
 #'
 #' @name scale_size
 #' @inheritParams continuous_scale
+#' @inheritParams binned_scale
 #' @param range a numeric vector of length 2 that specifies the minimum and
 #'   maximum size of the plotting symbol after transformation.
 #' @seealso [scale_size_area()] if you want 0 values to be mapped
@@ -22,6 +25,9 @@
 #' # If you want zero value to have zero size, use scale_size_area:
 #' p + scale_size_area()
 #'
+#' # Binning can sometimes make it easier to match the scaled data to the legend
+#' p + scale_size_binned()
+#'
 #' # This is most useful when size is a count
 #' ggplot(mpg, aes(class, cyl)) +
 #'   geom_count() +
@@ -42,6 +48,10 @@ scale_size_continuous <- function(name = waiver(), breaks = waiver(), labels = w
     guide = guide)
 }
 
+#' @rdname scale_size
+#' @export
+scale_size <- scale_size_continuous
+
 #' @rdname scale_size
 #' @export
 scale_radius <- function(name = waiver(), breaks = waiver(), labels = waiver(),
@@ -54,13 +64,19 @@ scale_radius <- function(name = waiver(), breaks = waiver(), labels = waiver(),
 
 #' @rdname scale_size
 #' @export
-scale_size <- scale_size_continuous
+scale_size_binned <- function(name = waiver(), breaks = waiver(), labels = waiver(),
+                              limits = NULL, range = c(1, 6), n.breaks = NULL,
+                              nice.breaks = TRUE, trans = "identity", guide = "bins") {
+  binned_scale("size", "area_b", area_pal(range), name = name,
+               breaks = breaks, labels = labels, limits = limits, trans = trans,
+               n.breaks = n.breaks, nice.breaks = nice.breaks, guide = guide)
+}
 
 #' @rdname scale_size
 #' @export
 #' @usage NULL
 scale_size_discrete <- function(...) {
-  warning("Using size for a discrete variable is not advised.", call. = FALSE)
+  warn("Using size for a discrete variable is not advised.")
   scale_size_ordinal(...)
 }
 
@@ -68,6 +84,8 @@ scale_size_discrete <- function(...) {
 #' @export
 #' @usage NULL
 scale_size_ordinal <- function(..., range = c(2, 6)) {
+  force(range)
+
   discrete_scale(
     "size",
     "size_d",
@@ -89,6 +107,14 @@ scale_size_area <- function(..., max_size = 6) {
     rescaler = rescale_max, ...)
 }
 
+#' @export
+#' @rdname scale_size
+scale_size_binned_area <- function(..., max_size = 6) {
+  binned_scale("size", "area_b",
+               palette = abs_area(max_size),
+               rescaler = rescale_max, ...)
+}
+
 #' @rdname scale_size
 #' @export
 #' @usage NULL
