@@ -21,6 +21,8 @@
 #'   \item{count}{density * number of points - useful for stacked density
 #'      plots}
 #'   \item{scaled}{density estimate, scaled to maximum of 1}
+#'   \item{ndensity}{alias for `scaled`, to mirror the syntax of
+#'    [`stat_bin()`]}
 #' }
 #' @export
 #' @rdname geom_density
@@ -33,6 +35,7 @@ stat_density <- function(mapping = NULL, data = NULL,
                          n = 512,
                          trim = FALSE,
                          na.rm = FALSE,
+                         orientation = NA,
                          show.legend = NA,
                          inherit.aes = TRUE) {
 
@@ -51,6 +54,7 @@ stat_density <- function(mapping = NULL, data = NULL,
       n = n,
       trim = trim,
       na.rm = na.rm,
+      orientation = orientation,
       ...
     )
   )
@@ -61,19 +65,37 @@ stat_density <- function(mapping = NULL, data = NULL,
 #' @usage NULL
 #' @export
 StatDensity <- ggproto("StatDensity", Stat,
-  required_aes = "x",
-  default_aes = aes(y = stat(density), fill = NA),
+  required_aes = "x|y",
+
+  default_aes = aes(x = after_stat(density), y = after_stat(density), fill = NA, weight = NULL),
+
+  setup_params = function(data, params) {
+    params$flipped_aes <- has_flipped_aes(data, params, main_is_orthogonal = FALSE, main_is_continuous = TRUE)
+
+    has_x <- !(is.null(data$x) && is.null(params$x))
+    has_y <- !(is.null(data$y) && is.null(params$y))
+    if (!has_x && !has_y) {
+      abort("stat_density() requires an x or y aesthetic.")
+    }
+
+    params
+  },
+
+  extra_params = c("na.rm", "orientation"),
 
   compute_group = function(data, scales, bw = "nrd0", adjust = 1, kernel = "gaussian",
-                           n = 512, trim = FALSE, na.rm = FALSE) {
+                           n = 512, trim = FALSE, na.rm = FALSE, flipped_aes = FALSE) {
+    data <- flip_data(data, flipped_aes)
     if (trim) {
       range <- range(data$x, na.rm = TRUE)
     } else {
-      range <- scales$x$dimension()
+      range <- scales[[flipped_names(flipped_aes)$x]]$dimension()
     }
 
-    compute_density(data$x, data$weight, from = range[1], to = range[2],
-      bw = bw, adjust = adjust, kernel = kernel, n = n)
+    density <- compute_density(data$x, data$weight, from = range[1],
+      to = range[2], bw = bw, adjust = adjust, kernel = kernel, n = n)
+    density$flipped_aes <- flipped_aes
+    flip_data(density, flipped_aes)
   }
 
 )
@@ -83,28 +105,32 @@ compute_density <- function(x, w, from, to, bw = "nrd0", adjust = 1,
   nx <- length(x)
   if (is.null(w)) {
     w <- rep(1 / nx, nx)
+  } else {
+    w <- w / sum(w)
   }
 
   # if less than 2 points return data frame of NAs and a warning
   if (nx < 2) {
-    warning("Groups with fewer than two data points have been dropped.", call. = FALSE)
-    return(data.frame(
+    warn("Groups with fewer than two data points have been dropped.")
+    return(new_data_frame(list(
       x = NA_real_,
       density = NA_real_,
       scaled = NA_real_,
+      ndensity = NA_real_,
       count = NA_real_,
       n = NA_integer_
-    ))
+    ), n = 1))
   }
 
   dens <- stats::density(x, weights = w, bw = bw, adjust = adjust,
     kernel = kernel, n = n, from = from, to = to)
 
-  data.frame(
+  new_data_frame(list(
     x = dens$x,
     density = dens$y,
     scaled =  dens$y / max(dens$y, na.rm = TRUE),
+    ndensity = dens$y / max(dens$y, na.rm = TRUE),
     count =   dens$y * nx,
     n = nx
-  )
+  ), n = length(dens$x))
 }
