@@ -1,66 +1,84 @@
 #' Summarise y values at unique/binned x
 #'
-#' `stat_summary` operates on unique `x`; `stat_summary_bin`
-#' operates on binned `x`. They are more flexible versions of
+#' `stat_summary` operates on unique `x` or `y`; `stat_summary_bin`
+#' operates on binned `x` or `y`. They are more flexible versions of
 #' [stat_bin()]: instead of just counting, they can compute any
 #' aggregate.
 #'
+#' @eval rd_orientation()
+#'
 #' @eval rd_aesthetics("stat", "summary")
 #' @seealso [geom_errorbar()], [geom_pointrange()],
 #'  [geom_linerange()], [geom_crossbar()] for geoms to
 #'  display summarised data
 #' @inheritParams stat_identity
 #' @section Summary functions:
-#' You can either supply summary functions individually (`fun.y`,
-#' `fun.ymax`, `fun.ymin`), or as a single function (`fun.data`):
+#' You can either supply summary functions individually (`fun`,
+#' `fun.max`, `fun.min`), or as a single function (`fun.data`):
 #'
 #' \describe{
 #'   \item{fun.data}{Complete summary function. Should take numeric vector as
 #'      input and return data frame as output}
-#'   \item{fun.ymin}{ymin summary function (should take numeric vector and
+#'   \item{fun.min}{min summary function (should take numeric vector and
 #'     return single number)}
-#'   \item{fun.y}{y summary function (should take numeric vector and return
+#'   \item{fun}{main summary function (should take numeric vector and return
 #'     single number)}
-#'   \item{fun.ymax}{ymax summary function (should take numeric vector and
+#'   \item{fun.max}{max summary function (should take numeric vector and
 #'     return single number)}
 #' }
 #'
 #' A simple vector function is easiest to work with as you can return a single
 #' number, but is somewhat less flexible. If your summary function computes
-#' multiple values at once (e.g. ymin and ymax), use `fun.data`.
+#' multiple values at once (e.g. min and max), use `fun.data`.
+#'
+#' `fun.data` will recieve data as if it was oriented along the x-axis and
+#' should return a data.frame that corresponds to that orientation. The layer
+#' will take care of flipping the input and output if it is oriented along the
+#' y-axis.
 #'
 #' If no aggregation functions are supplied, will default to
 #' [mean_se()].
 #'
 #' @param fun.data A function that is given the complete data and should
 #'   return a data frame with variables `ymin`, `y`, and `ymax`.
-#' @param fun.ymin,fun.y,fun.ymax Alternatively, supply three individual
-#'   functions that are each passed a vector of x's and should return a
+#' @param fun.min,fun,fun.max Alternatively, supply three individual
+#'   functions that are each passed a vector of values and should return a
 #'   single number.
+#' @param fun.ymin,fun.y,fun.ymax Deprecated, use the versions specified above
+#'   instead.
 #' @param fun.args Optional additional arguments passed on to the functions.
 #' @export
 #' @examples
 #' d <- ggplot(mtcars, aes(cyl, mpg)) + geom_point()
 #' d + stat_summary(fun.data = "mean_cl_boot", colour = "red", size = 2)
 #'
+#' # Orientation follows the discrete axis
+#' ggplot(mtcars, aes(mpg, cyl)) +
+#'   geom_point() +
+#'   stat_summary(fun.data = "mean_cl_boot", colour = "red", size = 2)
+#'
 #' # You can supply individual functions to summarise the value at
 #' # each x:
-#' d + stat_summary(fun.y = "median", colour = "red", size = 2, geom = "point")
-#' d + stat_summary(fun.y = "mean", colour = "red", size = 2, geom = "point")
-#' d + aes(colour = factor(vs)) + stat_summary(fun.y = mean, geom="line")
+#' d + stat_summary(fun = "median", colour = "red", size = 2, geom = "point")
+#' d + stat_summary(fun = "mean", colour = "red", size = 2, geom = "point")
+#' d + aes(colour = factor(vs)) + stat_summary(fun = mean, geom="line")
 #'
-#' d + stat_summary(fun.y = mean, fun.ymin = min, fun.ymax = max,
+#' d + stat_summary(fun = mean, fun.min = min, fun.max = max,
 #'   colour = "red")
 #'
 #' d <- ggplot(diamonds, aes(cut))
 #' d + geom_bar()
-#' d + stat_summary_bin(aes(y = price), fun.y = "mean", geom = "bar")
+#' d + stat_summary(aes(y = price), fun = "mean", geom = "bar")
+#'
+#' # Orientation of stat_summary_bin is ambiguous and must be specified directly
+#' ggplot(diamonds, aes(carat, price)) +
+#'   stat_summary_bin(fun = "mean", geom = "bar", orientation = 'y')
 #'
 #' \donttest{
 #' # Don't use ylim to zoom into a summary plot - this throws the
 #' # data away
 #' p <- ggplot(mtcars, aes(cyl, mpg)) +
-#'   stat_summary(fun.y = "mean", geom = "point")
+#'   stat_summary(fun = "mean", geom = "point")
 #' p
 #' p + ylim(15, 30)
 #' # Instead use coord_cartesian
@@ -105,13 +123,27 @@ stat_summary <- function(mapping = NULL, data = NULL,
                          geom = "pointrange", position = "identity",
                          ...,
                          fun.data = NULL,
-                         fun.y = NULL,
-                         fun.ymax = NULL,
-                         fun.ymin = NULL,
+                         fun = NULL,
+                         fun.max = NULL,
+                         fun.min = NULL,
                          fun.args = list(),
                          na.rm = FALSE,
+                         orientation = NA,
                          show.legend = NA,
-                         inherit.aes = TRUE) {
+                         inherit.aes = TRUE,
+                         fun.y, fun.ymin, fun.ymax) {
+  if (!missing(fun.y)) {
+    warn("`fun.y` is deprecated. Use `fun` instead.")
+    fun = fun %||% fun.y
+  }
+  if (!missing(fun.ymin)) {
+    warn("`fun.ymin` is deprecated. Use `fun.min` instead.")
+    fun.min = fun.min %||% fun.ymin
+  }
+  if (!missing(fun.ymax)) {
+    warn("`fun.ymax` is deprecated. Use `fun.max` instead.")
+    fun.max = fun.max %||% fun.ymax
+  }
   layer(
     data = data,
     mapping = mapping,
@@ -122,11 +154,12 @@ stat_summary <- function(mapping = NULL, data = NULL,
     inherit.aes = inherit.aes,
     params = list(
       fun.data = fun.data,
-      fun.y = fun.y,
-      fun.ymax = fun.ymax,
-      fun.ymin = fun.ymin,
+      fun = fun,
+      fun.max = fun.max,
+      fun.min = fun.min,
       fun.args = fun.args,
       na.rm = na.rm,
+      orientation = orientation,
       ...
     )
   )
@@ -139,12 +172,20 @@ stat_summary <- function(mapping = NULL, data = NULL,
 StatSummary <- ggproto("StatSummary", Stat,
   required_aes = c("x", "y"),
 
-  compute_panel = function(data, scales, fun.data = NULL, fun.y = NULL,
-                     fun.ymax = NULL, fun.ymin = NULL, fun.args = list(),
-                     na.rm = FALSE) {
+  extra_params = c("na.rm", "orientation"),
+  setup_params = function(data, params) {
+    params$flipped_aes <- has_flipped_aes(data, params)
+    params
+  },
 
-    fun <- make_summary_fun(fun.data, fun.y, fun.ymax, fun.ymin, fun.args)
-    summarise_by_x(data, fun)
+  compute_panel = function(data, scales, fun.data = NULL, fun = NULL,
+                     fun.max = NULL, fun.min = NULL, fun.args = list(),
+                     na.rm = FALSE, flipped_aes = FALSE) {
+    data <- flip_data(data, flipped_aes)
+    fun <- make_summary_fun(fun.data, fun, fun.max, fun.min, fun.args)
+    summarised <- summarise_by_x(data, fun)
+    summarised$flipped_aes <- flipped_aes
+    flip_data(summarised, flipped_aes)
   }
 )
 
@@ -161,8 +202,8 @@ StatSummary <- ggproto("StatSummary", Stat,
 # @param other arguments passed on to summary function
 # @keyword internal
 summarise_by_x <- function(data, summary, ...) {
-  summary <- plyr::ddply(data, c("group", "x"), summary, ...)
-  unique <- plyr::ddply(data, c("group", "x"), uniquecols)
+  summary <- dapply(data, c("group", "x"), summary, ...)
+  unique <- dapply(data, c("group", "x"), uniquecols)
   unique$y <- NULL
 
   merge(summary, unique, by = c("x", "group"), sort = FALSE)
@@ -196,15 +237,14 @@ wrap_hmisc <- function(fun) {
 
   function(x, ...) {
     if (!requireNamespace("Hmisc", quietly = TRUE))
-      stop("Hmisc package required for this function", call. = FALSE)
+      abort("Hmisc package required for this function")
 
     fun <- getExportedValue("Hmisc", fun)
     result <- do.call(fun, list(x = quote(x), ...))
 
-    plyr::rename(
-      data.frame(t(result)),
-      c(Median = "y", Mean = "y", Lower = "ymin", Upper = "ymax"),
-      warn_missing = FALSE
+    rename(
+      new_data_frame(as.list(result)),
+      c(Median = "y", Mean = "y", Lower = "ymin", Upper = "ymax")
     )
   }
 }
@@ -236,5 +276,5 @@ mean_se <- function(x, mult = 1) {
   x <- stats::na.omit(x)
   se <- mult * sqrt(stats::var(x) / length(x))
   mean <- mean(x)
-  data.frame(y = mean, ymin = mean - se, ymax = mean + se)
+  new_data_frame(list(y = mean, ymin = mean - se, ymax = mean + se), n = 1)
 }
