@@ -22,7 +22,7 @@
 #'
 #' \item{`theme_linedraw`}{
 #' A theme with only black lines of various widths on white backgrounds,
-#' reminiscent of a line drawings. Serves a purpose similar to `theme_bw`.
+#' reminiscent of a line drawing. Serves a purpose similar to `theme_bw`.
 #' Note that this theme has some very thin lines (<< 1 pt) which some journals
 #' may refuse.}
 #'
@@ -111,7 +111,7 @@ theme_grey <- function(base_size = 11, base_family = "",
   # Throughout the theme, we use three font sizes, `base_size` (`rel(1)`)
   # for normal, `rel(0.8)` for small, and `rel(1.2)` for large.
 
-  theme(
+  t <- theme(
     # Elements in this first block aren't used directly, but are inherited
     # by others
     line =               element_line(
@@ -169,7 +169,7 @@ theme_grey <- function(base_size = 11, base_family = "",
     legend.spacing.x =    NULL,
     legend.spacing.y =    NULL,
     legend.margin =      margin(half_line, half_line, half_line, half_line),
-    legend.key =         element_rect(fill = "grey95", colour = "white"),
+    legend.key =         element_rect(fill = "grey95", colour = NA),
     legend.key.size =    unit(1.2, "lines"),
     legend.key.height =  NULL,
     legend.key.width =   NULL,
@@ -202,6 +202,7 @@ theme_grey <- function(base_size = 11, base_family = "",
                          ),
     strip.text.x =       NULL,
     strip.text.y =       element_text(angle = -90),
+    strip.text.y.left =  element_text(angle = 90),
     strip.placement =    "inside",
     strip.placement.x =  NULL,
     strip.placement.y =  NULL,
@@ -214,6 +215,7 @@ theme_grey <- function(base_size = 11, base_family = "",
                            hjust = 0, vjust = 1,
                            margin = margin(b = half_line)
                          ),
+    plot.title.position = "panel",
     plot.subtitle =      element_text( # font size "regular"
                            hjust = 0, vjust = 1,
                            margin = margin(b = half_line)
@@ -223,6 +225,7 @@ theme_grey <- function(base_size = 11, base_family = "",
                            hjust = 1, vjust = 1,
                            margin = margin(t = half_line)
                          ),
+    plot.caption.position = "panel",
     plot.tag =           element_text(
                            size = rel(1.2),
                            hjust = 0.5, vjust = 0.5
@@ -232,6 +235,9 @@ theme_grey <- function(base_size = 11, base_family = "",
 
     complete = TRUE
   )
+
+  # make sure all elements are set to NULL if not explicitly defined
+  ggplot_global$theme_all_null %+replace% t
 }
 #' @export
 #' @rdname ggtheme
@@ -453,7 +459,7 @@ theme_void <- function(base_size = 11, base_family = "",
   half_line <- base_size / 2
 
   # Only keep indispensable text: legend and plot titles
-  theme(
+  t <- theme(
     line =               element_blank(),
     rect =               element_blank(),
     text =               element_text(
@@ -487,6 +493,7 @@ theme_void <- function(base_size = 11, base_family = "",
                            hjust = 0, vjust = 1,
                            margin = margin(t = half_line)
                          ),
+    plot.title.position = "panel",
     plot.subtitle =      element_text(
                            hjust = 0, vjust = 1,
                            margin = margin(t = half_line)
@@ -496,6 +503,7 @@ theme_void <- function(base_size = 11, base_family = "",
                            hjust = 1, vjust = 1,
                            margin = margin(t = half_line)
                          ),
+    plot.caption.position = "panel",
     plot.tag =           element_text(
                            size = rel(1.2),
                            hjust = 0.5, vjust = 0.5
@@ -504,6 +512,9 @@ theme_void <- function(base_size = 11, base_family = "",
 
     complete = TRUE
   )
+
+  # make sure all elements are set to NULL if not explicitly defined
+  ggplot_global$theme_all_null %+replace% t
 }
 
 
@@ -514,7 +525,7 @@ theme_test <- function(base_size = 11, base_family = "",
                        base_rect_size = base_size / 22) {
   half_line <- base_size / 2
 
-  theme(
+  t <- theme(
     line =               element_line(
                            colour = "black", size = base_line_size,
                            linetype = 1, lineend = "butt"
@@ -615,6 +626,7 @@ theme_test <- function(base_size = 11, base_family = "",
                            hjust = 0, vjust = 1,
                            margin = margin(b = half_line)
                          ),
+    plot.title.position = "panel",
     plot.subtitle =      element_text(
                            hjust = 0, vjust = 1,
                            margin = margin(b = half_line)
@@ -624,6 +636,7 @@ theme_test <- function(base_size = 11, base_family = "",
                            hjust = 1, vjust = 1,
                            margin = margin(t = half_line)
                          ),
+    plot.caption.position = "panel",
     plot.tag =           element_text(
                            size = rel(1.2),
                            hjust = 0.5, vjust = 0.5
@@ -633,4 +646,23 @@ theme_test <- function(base_size = 11, base_family = "",
 
     complete = TRUE
   )
+
+  # make sure all elements are set to NULL if not explicitly defined
+  ggplot_global$theme_all_null %+replace% t
+}
+
+theme_all_null <- function() {
+  # Set all elements in the element tree to NULL.
+
+  # We read from `.element_tree` instead of `ggplot_global$element_tree`
+  # because we don't want to change our results just because a user
+  # has defined new theme elements.
+  elements <- sapply(
+    names(.element_tree),
+    function(x) NULL,
+    simplify = FALSE, USE.NAMES = TRUE
+  )
+
+  args <- c(elements, list(complete = TRUE))
+  do.call(theme, args)
 }
